@@ -136,3 +136,59 @@ def test_create_bigquery_client_with_default_credentials_respects_project(
     )
 
     assert bqclient.project == "connection-url-project"
+
+
+def test_substitute_re_string(module_under_test):
+    import re
+
+    foo_to_baz = module_under_test.substitute_re_method("foo", re.IGNORECASE, "baz")
+    assert (
+        foo_to_baz(object(), "some foo and FOO is good") == "some baz and baz is good"
+    )
+
+
+def test_substitute_re_func(module_under_test):
+    import re
+
+    @module_under_test.substitute_re_method("foo", re.IGNORECASE)
+    def Foo_to_bar(self, m):
+        return "bar"
+
+    assert (
+        Foo_to_bar(object(), "some foo and FOO is good") == "some bar and bar is good"
+    )
+
+    @module_under_test.substitute_re_method("foo")
+    def foo_to_bar(self, m, x="bar"):
+        return x
+
+    assert (
+        foo_to_bar(object(), "some foo and FOO is good") == "some bar and FOO is good"
+    )
+
+    assert (
+        foo_to_bar(object(), "some foo and FOO is good", "hah")
+        == "some hah and FOO is good"
+    )
+
+    assert (
+        foo_to_bar(object(), "some foo and FOO is good", x="hah")
+        == "some hah and FOO is good"
+    )
+
+    assert foo_to_bar.__name__ == "foo_to_bar"
+
+
+def test_substitute_re_func_self(module_under_test):
+    class Replacer:
+        def __init__(self, x):
+            self.x = x
+
+        @module_under_test.substitute_re_method("foo")
+        def foo_to_bar(self, m):
+            return self.x
+
+    assert (
+        Replacer("hah").foo_to_bar("some foo and FOO is good")
+        == "some hah and FOO is good"
+    )
